//	(c) 2005 Philipp Krause pkk@spth.de

//	This library is free software; you can redistribute it and/or modify it
//	under the terms of the GNU Library General Public License as published by the
//	Free Software Foundation; either version 2, or (at your option) any
//	later version.
   
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU Library General Public License for more details.
   
//	You should have received a copy of the GNU Library General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "cv.h"
#include "cvu_sound.h"

const uint16_t CVU_TUNING_ISO16_EQUAL[15]	// ISO 16 pitch (A at 440 Hz) with equal tuning.
= {54719, 51648, 48749, 46013, 43431, 40993, 38692, 36521, 34471, 32536, 30710, 28987, 0, 0, 0};

const uint16_t CVU_TUNING_SCIENTIFIC_EQUAL[15]	// Scientific pitch (C at 256 Hz) with equal tuning.
= {55922, 52783, 49821, 47025, 44385, 41894, 39543, 37323, 35229, 33251, 31385, 29624, 0, 0, 0};

// Todo: Adjust this!
const uint8_t CVU_VOLUME_DEFAULT[4] = {8, 12, 16, 20};

const uint16_t CVU_EMPTY_MUSIC = 0xffff;

void cvu_init_music(struct cvu_music *music)
{
	music->channel = CV_SOUNDCHANNEL_0;
	music->volume = CVU_VOLUME_DEFAULT;
	music->tuning = CVU_TUNING_ISO16_EQUAL;
	music->sixteenth_notes_per_second = 10;
	music->note_vint_remaining = 0;
	music->pause_vint_remaining = 0;
	music->notes = &CVU_EMPTY_MUSIC;
}

bool cvu_play_music(struct cvu_music *music)
{
	if(music->note_vint_remaining)
	{
		music->note_vint_remaining--;
	}
	else if(music->pause_vint_remaining)
	{
		cv_set_attenuation(music->channel, 0);
		music->pause_vint_remaining--;
	}
	else
	{
		bool pause = false;
		const uint16_t note = *(music->notes);

		cv_set_attenuation(music->channel, 0);

		if(note== 0xffff)
			return(false);

		// Length calculations:
		{
			uint8_t length, rel_length;

			length = (note >> 4) & 0xf;
			if(!length)
				length = 0x10;
			music->note_vint_remaining = length;
			music->note_vint_remaining *= cv_get_vint_frequency();
			music->note_vint_remaining /= music->sixteenth_notes_per_second;
			music->note_vint_remaining--;

			rel_length = (note >> 2) & 0x3;
			switch(rel_length)
			{
			case 0:	// Legato
				break;
			case 1:	// Staccato
				music->pause_vint_remaining = music->note_vint_remaining;
				music->note_vint_remaining = music->note_vint_remaining >> 2;
				music->pause_vint_remaining -= music->note_vint_remaining;
				break;
			case 2:
				music->note_vint_remaining = music->note_vint_remaining >> 1;
				music->pause_vint_remaining = music->note_vint_remaining;
				break;
			default:	// Standard
				music->pause_vint_remaining = music->note_vint_remaining >> 2;
				music->note_vint_remaining -= music->pause_vint_remaining;
				break;
			}
		}
		
		// Frequency calculations:
		{
			uint8_t octave, halftone;
			uint16_t frequency_divider;
			
			halftone = (note >> 8) & 0xf;
			pause = (halftone == 0xf);
			if(!pause)
			{
				frequency_divider = music->tuning[halftone];

				octave = (note >> 12);
				cv_set_frequency(music->channel, (frequency_divider >> octave) <= 32736 ? frequency_divider >> octave : frequency_divider >> (octave + 1));
			}
		}

		// Loudness calculations:
		{
			cv_set_attenuation(music->channel, pause ? 0 : (music->volume[note & 0x3]));
		}
		music->notes++;
	}
	return(true);
}
